@@ -24,6 +24,7 @@
 #include <assert.h>
 #include <limits.h>
 #include <stddef.h> /* for offsetof */
+#include <string.h>
 
 #include <proj_api.h>
 
@@ -40,6 +41,25 @@
 #include "readval.h"
 #include "str.h"
 
+/*** Extracted from proj.4 projects (yuck, but grass also does this): */
+struct DERIVS {
+    double x_l, x_p; /* derivatives of x for lambda-phi */
+    double y_l, y_p; /* derivatives of y for lambda-phi */
+};
+
+struct FACTORS {
+    struct DERIVS der;
+    double h, k;	/* meridinal, parallel scales */
+    double omega, thetap;	/* angular distortion, theta prime */
+    double conv;	/* convergence */
+    double s;		/* areal scale factor */
+    double a, b;	/* max-min scale error */
+    int code;		/* info as to analytics, see following */
+};
+
+int pj_factors(projLP, projPJ *, double, struct FACTORS *);
+/***/
+
 static projPJ proj_wgs84;
 
 static void
@@ -666,9 +686,8 @@ cmd_begin(void)
    pcs = pcsNew;
 
    skipblanks();
-   if (isEol(ch) || isComm(ch)) {
-      pcs->begin_survey = NULL;
-   } else {
+   pcs->begin_survey = NULL;
+   if (!isEol(ch) && !isComm(ch)) {
       prefix *survey = read_prefix(PFX_SURVEY|PFX_ALLOW_ROOT|PFX_WARN_SEPARATOR);
       pcs->begin_survey = survey;
       pcs->Prefix = survey;
@@ -1646,6 +1665,15 @@ cmd_declination(void)
 	pcs->dec_x = x;
 	pcs->dec_y = y;
 	pcs->dec_z = z;
+	/* Invalidate cached declination. */
+	pcs->declination = HUGE_REAL;
+	{
+	    projLP lp = { x, y };
+	    struct FACTORS factors;
+	    memset(&factors, 0, sizeof(factors));
+	    pj_factors(lp, proj_out, 0.0, &factors);
+	    pcs->convergence = factors.conv;
+	}
     } else {
 	/* *declination D UNITS */
 	int units = get_units(BIT(Q_DECLINATION), fFalse);
@@ -1653,6 +1681,7 @@ cmd_declination(void)
 	    return;
 	}
 	pcs->z[Q_DECLINATION] = -v * factor_tab[units];
+	pcs->convergence = 0;
     }
 }
 
@@ -2208,7 +2237,8 @@ cmd_require(void)
       }
       if (ch != '.') break;
       nextch();
-      if (!isdigit(ch) || ver == version + sizeof(version)) break;
+      if (!isdigit(ch) || ver == version + sizeof(version) / sizeof(*version))
+	 break;
    }
    /* skip rest of version number */
    while (isdigit(ch) || ch == '.') nextch();
