@@ -1,6 +1,6 @@
 /* dump3d.c */
 /* Show raw contents of .3d file in text form */
-/* Copyright (C) 2001,2002,2006,2011,2012,2013,2014 Olly Betts
+/* Copyright (C) 2001,2002,2006,2011,2012,2013,2014,2015 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -76,19 +76,6 @@ main(int argc, char **argv)
    }
    fnm = argv[optind];
 
-#if 0
-   pimg = img_open_write("dump3d.3d", "dump3d", 0);
-   if (!pimg) fatalerror(img_error2msg(img_error()), "dump3d.3d");
-   img_write_item(pimg, img_MOVE, 0, NULL, 0, 0, 0);
-   img_write_item(pimg, img_LINE, 0, "161.lostworld.upstream",
-		  36920.120000, 82672.840000, 1528.820000);
-   img_write_item(pimg, img_LINE, 0, "161.keinzimmer",
-		  36918.480000, 82663.360000, 1527.670000);
-   img_write_item(pimg, img_LINE, 0, "161.lostworld.upstream",
-		  36959.760000, 82679.880000, 1537.900000);
-   img_close(pimg);
-#endif
-
    pimg = img_open_survey(fnm, survey);
    if (!pimg) fatalerror(img_error2msg(img_error()), fnm);
 
@@ -100,6 +87,7 @@ main(int argc, char **argv)
    printf("VERSION %d\n", pimg->version);
    if (pimg->is_extended_elevation)
       printf("EXTENDED ELEVATION\n");
+   printf("SEPARATOR '%c'\n", pimg->separator);
    printf("--\n");
 
    code = img_BAD;
