@@ -2,7 +2,7 @@
  * Export to CAD-like formats (DXF, Skencil, SVG, EPS) and also Compass PLT.
  */
 
-/* Copyright (C) 1994-2004,2005,2006,2008,2010,2011,2012,2013,2014,2015 Olly Betts
+/* Copyright (C) 1994-2004,2005,2006,2008,2010,2011,2012,2013,2014,2015,2016 Olly Betts
  * Copyright (C) 2004 John Pybus (SVG Output code)
  *
  * This program is free software; you can redistribute it and/or modify
@@ -89,6 +89,7 @@ html_escape(FILE *fh, const char *s)
     }
 }
 
+// Used by Skencil and SVG.
 static const char *layer_name(int mask) {
     switch (mask) {
 	case LEGS: case LEGS|SURF:
@@ -129,11 +130,11 @@ class DXF : public ExportFilter {
     explicit DXF(double text_height_)
 	: to_close(0), text_height(text_height_) { pending[0] = '\0'; }
     const int * passes() const;
-    bool fopen(const char *fnm_out);
+    bool fopen(const wxString& fnm_out);
     void header(const char *, const char *, time_t,
 		double min_x, double min_y, double min_z,
 		double max_x, double max_y, double max_z);
-    void line(const img_point *, const img_point *, bool, bool);
+    void line(const img_point *, const img_point *, unsigned, bool);
     void label(const img_point *, const char *, bool, int);
     void cross(const img_point *, bool);
     void xsect(const img_point *, double, double, double);
@@ -153,10 +154,10 @@ DXF::passes() const
 }
 
 bool
-DXF::fopen(const char *fnm_out)
+DXF::fopen(const wxString& fnm_out)
 {
     // DXF gets written as text rather than binary.
-    fh = ::fopen(fnm_out, "w");
+    fh = wxFopen(fnm_out.fn_str(), wxT("w"));
     return (fh != NULL);
 }
 
@@ -282,8 +283,9 @@ DXF::header(const char *, const char *, time_t,
 }
 
 void
-DXF::line(const img_point *p1, const img_point *p, bool fSurface, bool fPendingMove)
+DXF::line(const img_point *p1, const img_point *p, unsigned flags, bool fPendingMove)
 {
+   bool fSurface = (flags & SURF);
    (void)fPendingMove; /* unused */
    fprintf(fh, "0\nLINE\n");
    fprintf(fh, fSurface ? "8\nSurface\n" : "8\nCentreLine\n"); /* Layer */
@@ -403,7 +405,7 @@ class Skencil : public ExportFilter {
 		double min_x, double min_y, double min_z,
 		double max_x, double max_y, double max_z);
     void start_pass(int layer);
-    void line(const img_point *, const img_point *, bool, bool);
+    void line(const img_point *, const img_point *, unsigned, bool);
     void label(const img_point *, const char *, bool, int);
     void cross(const img_point *, bool);
     void footer();
@@ -434,9 +436,9 @@ Skencil::start_pass(int layer)
 }
 
 void
-Skencil::line(const img_point *p1, const img_point *p, bool fSurface, bool fPendingMove)
+Skencil::line(const img_point *p1, const img_point *p, unsigned flags, bool fPendingMove)
 {
-   (void)fSurface; /* unused */
+   (void)flags; /* unused */
    if (fPendingMove) {
        fprintf(fh, "b()\n");
        fprintf(fh, "bs(%.3f,%.3f,%.3f)\n", p1->x * factor, p1->y * factor, 0.0);
@@ -537,7 +539,7 @@ find_name(const img_point *p)
       char data[sizeof(int) * 3];
       int x[3];
    } u;
-   SVX_ASSERT(p);
+   wxASSERT(p);
 
    u.x[0] = (int)(p->x * 100);
    u.x[1] = (int)(p->y * 100);
@@ -571,7 +573,7 @@ class SVG : public ExportFilter {
 		double min_x, double min_y, double min_z,
 		double max_x, double max_y, double max_z);
     void start_pass(int layer);
-    void line(const img_point *, const img_point *, bool, bool);
+    void line(const img_point *, const img_point *, unsigned, bool);
     void label(const img_point *, const char *, bool, int);
     void cross(const img_point *, bool);
     void xsect(const img_point *, double, double, double);
@@ -649,14 +651,16 @@ SVG::start_pass(int layer)
 }
 
 void
-SVG::line(const img_point *p1, const img_point *p, bool fSurface, bool fPendingMove)
+SVG::line(const img_point *p1, const img_point *p, unsigned flags, bool fPendingMove)
 {
-   (void)fSurface; /* unused */
+   bool splay = (flags & SPLAYS);
    if (fPendingMove) {
        if (to_close) {
 	   fputs(to_close, fh);
        }
-       fprintf(fh, "<path d=\"M%.3f %.3f", p1->x * factor, p1->y * -factor);
+       fprintf(fh, "<path ");
+       if (splay) fprintf(fh, "stroke=\"grey\" stroke-width=\"0.1px\" ");
+       fprintf(fh, "d=\"M%.3f %.3f", p1->x * factor, p1->y * -factor);
    }
    fprintf(fh, "L%.3f %.3f", p->x * factor, p->y * -factor);
    to_close = "\"/>\n";
@@ -763,7 +767,7 @@ class PLT : public ExportFilter {
     void header(const char *, const char *, time_t,
 		double min_x, double min_y, double min_z,
 		double max_x, double max_y, double max_z);
-    void line(const img_point *, const img_point *, bool, bool);
+    void line(const img_point *, const img_point *, unsigned, bool);
     void label(const img_point *, const char *, bool, int);
     void footer();
 };
@@ -798,9 +802,9 @@ PLT::header(const char *title, const char *, time_t,
 }
 
 void
-PLT::line(const img_point *p1, const img_point *p, bool fSurface, bool fPendingMove)
+PLT::line(const img_point *p1, const img_point *p, unsigned flags, bool fPendingMove)
 {
-   (void)fSurface; /* unused */
+   (void)flags; /* unused */
    if (fPendingMove) {
        /* Survex is E, N, Alt - PLT file is N, E, Alt */
        fprintf(fh, "M %.3f %.3f %.3f ",
@@ -871,7 +875,7 @@ class EPS : public ExportFilter {
 		double min_x, double min_y, double min_z,
 		double max_x, double max_y, double max_z);
     void start_pass(int layer);
-    void line(const img_point *, const img_point *, bool, bool);
+    void line(const img_point *, const img_point *, unsigned, bool);
     void label(const img_point *, const char *, bool, int);
     void cross(const img_point *, bool);
     void xsect(const img_point *, double, double, double);
@@ -1142,9 +1146,9 @@ EPS::start_pass(int layer)
 }
 
 void
-EPS::line(const img_point *p1, const img_point *p, bool fSurface, bool fPendingMove)
+EPS::line(const img_point *p1, const img_point *p, unsigned flags, bool fPendingMove)
 {
-   (void)fSurface; /* unused */
+   (void)flags; /* unused */
    if (fPendingMove) {
        fprintf(fh, "%.2f %.2f M\n", p1->x, p1->y);
    }
@@ -1248,6 +1252,25 @@ class UseNumericCLocale {
     }
 };
 
+static void
+transform_point(const Point& pos, const Vector3* pre_offset,
+		double COS, double SIN, double COST, double SINT,
+		img_point* p)
+{
+    double x = pos.GetX();
+    double y = pos.GetY();
+    double z = pos.GetZ();
+    if (pre_offset) {
+	x += pre_offset->GetX();
+	y += pre_offset->GetY();
+	z += pre_offset->GetZ();
+    }
+    p->x = x * COS - y * SIN;
+    double tmp = x * SIN + y * COS;
+    p->y = z * COST - tmp * SINT;
+    p->z = z * SINT + tmp * COST;
+}
+
 bool
 Export(const wxString &fnm_out, const wxString &title,
        const wxString &datestamp, time_t datestamp_numeric,
@@ -1260,9 +1283,11 @@ Export(const wxString &fnm_out, const wxString &title,
    UseNumericCLocale dummy;
    int fPendingMove = 0;
    img_point p, p1;
-   double s = 0, c = 0;
    const int *pass;
-   bool elevation = (tilt == 0.0);
+   double SIN = sin(rad(pan));
+   double COS = cos(rad(pan));
+   double SINT = sin(rad(tilt));
+   double COST = cos(rad(tilt));
 
    grid = grid_;
    marker_size = marker_size_;
@@ -1308,16 +1333,14 @@ Export(const wxString &fnm_out, const wxString &title,
 	   return false;
    }
 
-   // FIXME: This should really use fn_str() - currently we probably can't
-   // save to a Unicode path on wxmsw.
-   if (!filt->fopen(fnm_out.mb_str())) {
+   if (!filt->fopen(fnm_out)) {
        delete filt;
        return false;
    }
 
-   if (elevation) {
-      s = sin(rad(pan));
-      c = cos(rad(pan));
+   const Vector3* pre_offset = NULL;
+   if (show_mask & FULL_COORDS) {
+	pre_offset = &mainfrm->GetOffset();
    }
 
    /* Get bounding box */
@@ -1327,20 +1350,13 @@ Export(const wxString &fnm_out, const wxString &title,
    list<traverse>::const_iterator trav = mainfrm->traverses_begin();
    list<traverse>::const_iterator tend = mainfrm->traverses_end();
    for ( ; trav != tend; ++trav) {
+	if (trav->isSplay && (show_mask & SPLAYS) == 0) {
+	    continue;
+	}
 	vector<PointInfo>::const_iterator pos = trav->begin();
 	vector<PointInfo>::const_iterator end = trav->end();
 	for ( ; pos != end; ++pos) {
-	    p.x = pos->GetX();
-	    p.y = pos->GetY();
-	    p.z = pos->GetZ();
-
-	    if (elevation) {
-		double xnew = p.x * c - p.y * s;
-		double znew = - p.x * s - p.y * c;
-		p.y = p.z;
-		p.z = znew;
-		p.x = xnew;
-	    }
+	    transform_point(*pos, pre_offset, COS, SIN, COST, SINT, &p);
 
 	    if (p.x < min_x) min_x = p.x;
 	    if (p.x > max_x) max_x = p.x;
@@ -1354,17 +1370,7 @@ Export(const wxString &fnm_out, const wxString &title,
 	list<LabelInfo*>::const_iterator pos = mainfrm->GetLabels();
 	list<LabelInfo*>::const_iterator end = mainfrm->GetLabelsEnd();
 	for ( ; pos != end; ++pos) {
-	    p.x = (*pos)->GetX();
-	    p.y = (*pos)->GetY();
-	    p.z = (*pos)->GetZ();
-
-	    if (elevation) {
-		double xnew = p.x * c - p.y * s;
-		double znew = - p.x * s - p.y * c;
-		p.y = p.z;
-		p.z = znew;
-		p.x = xnew;
-	    }
+	    transform_point(**pos, pre_offset, COS, SIN, COST, SINT, &p);
 
 	    if (p.x < min_x) min_x = p.x;
 	    if (p.x > max_x) max_x = p.x;
@@ -1390,10 +1396,8 @@ Export(const wxString &fnm_out, const wxString &title,
 
    double x_offset, y_offset, z_offset;
    if (show_mask & FULL_COORDS) {
-       // Full coordinates.
-       x_offset = mainfrm->GetOffset().GetX();
-       y_offset = mainfrm->GetOffset().GetY();
-       z_offset = mainfrm->GetOffset().GetZ();
+       // Full coordinates - offset is applied before rotations.
+       x_offset = y_offset = z_offset = 0.0;
    } else if (show_mask & CENTRED) {
        // Centred.
        x_offset = (min_x + max_x) * -0.5;
@@ -1413,7 +1417,7 @@ Export(const wxString &fnm_out, const wxString &title,
    max_z += z_offset;
 
    /* Header */
-   filt->header(title.mb_str(), datestamp.mb_str(), datestamp_numeric,
+   filt->header(title.utf8_str(), datestamp.utf8_str(), datestamp_numeric,
 		min_x, min_y, min_z, max_x, max_y, max_z);
 
    p1.x = p1.y = p1.z = 0; /* avoid compiler warning */
@@ -1427,20 +1431,21 @@ Export(const wxString &fnm_out, const wxString &title,
 	  trav = mainfrm->traverses_begin();
 	  tend = mainfrm->traverses_end();
 	  for ( ; trav != tend; ++trav) {
+	     unsigned flags = 0;
+	     if (trav->isSplay) {
+		 if ((show_mask & SPLAYS) == 0) {
+		     continue;
+		 }
+		 flags = SPLAYS;
+	     }
 	     assert(trav->size() > 1);
 	     vector<PointInfo>::const_iterator pos = trav->begin();
 	     vector<PointInfo>::const_iterator end = trav->end();
 	     for ( ; pos != end; ++pos) {
-		 p.x = pos->GetX() + x_offset;
-		 p.y = pos->GetY() + y_offset;
-		 p.z = pos->GetZ() + z_offset;
-		 if (elevation) {
-		     double xnew = p.x * c - p.y * s;
-		     double znew = - p.x * s - p.y * c;
-		     p.y = p.z;
-		     p.z = znew;
-		     p.x = xnew;
-		 }
+		 transform_point(*pos, pre_offset, COS, SIN, COST, SINT, &p);
+		 p.x += x_offset;
+		 p.y += y_offset;
+		 p.z += z_offset;
 
 		 if (pos == trav->begin()) {
 		     // First point is move...
@@ -1452,7 +1457,7 @@ Export(const wxString &fnm_out, const wxString &title,
 #ifdef DEBUG_CAD3D
 		     printf("line to %9.2f %9.2f %9.2f\n", p.x, p.y, p.z);
 #endif
-		     filt->line(&p1, &p, false, fPendingMove);
+		     filt->line(&p1, &p, flags, fPendingMove);
 		     fPendingMove = 0;
 		 }
 		 p1 = p;
@@ -1463,21 +1468,21 @@ Export(const wxString &fnm_out, const wxString &title,
 	  trav = mainfrm->surface_traverses_begin();
 	  tend = mainfrm->surface_traverses_end();
 	  for ( ; trav != tend; ++trav) {
+	     unsigned flags = 0;
+	     if (trav->isSplay) {
+		 if ((show_mask & SPLAYS) == 0) {
+		     continue;
+		 }
+		 flags = SURF|SPLAYS;
+	     }
 	     assert(trav->size() > 1);
 	     vector<PointInfo>::const_iterator pos = trav->begin();
 	     vector<PointInfo>::const_iterator end = trav->end();
 	     for ( ; pos != end; ++pos) {
-		 p.x = pos->GetX() + x_offset;
-		 p.y = pos->GetY() + y_offset;
-		 p.z = pos->GetZ() + z_offset;
-
-		 if (elevation) {
-		     double xnew = p.x * c - p.y * s;
-		     double znew = - p.x * s - p.y * c;
-		     p.y = p.z;
-		     p.z = znew;
-		     p.x = xnew;
-		 }
+		 transform_point(*pos, pre_offset, COS, SIN, COST, SINT, &p);
+		 p.x += x_offset;
+		 p.y += y_offset;
+		 p.z += z_offset;
 
 		 if (pos == trav->begin()) {
 		     // First point is move...
@@ -1489,7 +1494,7 @@ Export(const wxString &fnm_out, const wxString &title,
 #ifdef DEBUG_CAD3D
 		     printf("surface line to %9.2f %9.2f %9.2f\n", p.x, p.y, p.z);
 #endif
-		     filt->line(&p1, &p, true, fPendingMove);
+		     filt->line(&p1, &p, flags, fPendingMove);
 		     fPendingMove = 0;
 		 }
 		 p1 = p;
@@ -1500,17 +1505,11 @@ Export(const wxString &fnm_out, const wxString &title,
 	  list<LabelInfo*>::const_iterator pos = mainfrm->GetLabels();
 	  list<LabelInfo*>::const_iterator end = mainfrm->GetLabelsEnd();
 	  for ( ; pos != end; ++pos) {
-	      p.x = (*pos)->GetX() + x_offset;
-	      p.y = (*pos)->GetY() + y_offset;
-	      p.z = (*pos)->GetZ() + z_offset;
+	      transform_point(**pos, pre_offset, COS, SIN, COST, SINT, &p);
+	      p.x += x_offset;
+	      p.y += y_offset;
+	      p.z += z_offset;
 
-	      if (elevation) {
-		  double xnew = p.x * c - p.y * s;
-		  double znew = - p.x * s - p.y * c;
-		  p.y = p.z;
-		  p.z = znew;
-		  p.x = xnew;
-	      }
 #ifdef DEBUG_CAD3D
 	      printf("label '%s' at %9.2f %9.2f %9.2f\n",(*pos)->GetText(),x,y,z);
 #endif
@@ -1530,13 +1529,14 @@ Export(const wxString &fnm_out, const wxString &title,
 	      bool f_surface = !(*pos)->IsUnderground();
 	      if (type) {
 		  const wxString & text = (*pos)->GetText();
-		  filt->label(&p, text.mb_str(), f_surface, type);
+		  filt->label(&p, text.utf8_str(), f_surface, type);
 	      }
 	      if (pass_mask & STNS)
 		  filt->cross(&p, f_surface);
 	  }
       }
       if (pass_mask & (XSECT|WALLS|PASG)) {
+	  bool elevation = (tilt == 0.0);
 	  list<vector<XSect> >::const_iterator tube = mainfrm->tubes_begin();
 	  list<vector<XSect> >::const_iterator tube_end = mainfrm->tubes_end();
 	  for ( ; tube != tube_end; ++tube) {
@@ -1544,16 +1544,12 @@ Export(const wxString &fnm_out, const wxString &title,
 	      vector<XSect>::const_iterator end = tube->end();
 	      for ( ; pos != end; ++pos) {
 		  const XSect & xs = *pos;
-		  p.x = xs.GetX() + x_offset;
-		  p.y = xs.GetY() + y_offset;
-		  p.z = xs.GetZ() + z_offset;
+		  transform_point(xs, pre_offset, COS, SIN, COST, SINT, &p);
+		  p.x += x_offset;
+		  p.y += y_offset;
+		  p.z += z_offset;
 
 		  if (elevation) {
-		      double xnew = p.x * c - p.y * s;
-		      double znew = - p.x * s - p.y * c;
-		      p.y = p.z;
-		      p.z = znew;
-		      p.x = xnew;
 		      if (pass_mask & XSECT)
 			  filt->xsect(&p, 90, xs.GetU(), xs.GetD());
 		      if (pass_mask & WALL1)
@@ -1563,14 +1559,16 @@ Export(const wxString &fnm_out, const wxString &title,
 		      if (pass_mask & PASG)
 			  filt->passage(&p, 90, xs.GetU(), xs.GetD());
 		  } else {
+		      // Should only be enabled in plan or elevation mode.
+		      double angle = pan + xs.get_right_bearing();
 		      if (pass_mask & XSECT)
-			  filt->xsect(&p, xs.get_right_bearing() + 180, xs.GetL(), xs.GetR());
+			  filt->xsect(&p, angle + 180, xs.GetL(), xs.GetR());
 		      if (pass_mask & WALL1)
-			  filt->wall(&p, xs.get_right_bearing() + 180, xs.GetL());
+			  filt->wall(&p, angle + 180, xs.GetL());
 		      if (pass_mask & WALL2)
-			  filt->wall(&p, xs.get_right_bearing(), xs.GetR());
+			  filt->wall(&p, angle, xs.GetR());
 		      if (pass_mask & PASG)
-			  filt->passage(&p, xs.get_right_bearing() + 180, xs.GetL(), xs.GetR());
+			  filt->passage(&p, angle + 180, xs.GetL(), xs.GetR());
 		  }
 	      }
 	      filt->tube_end();
