@@ -4,7 +4,7 @@
 //  Core drawing code for Aven.
 //
 //  Copyright (C) 2000-2001,2002,2005 Mark R. Shinwell.
-//  Copyright (C) 2001-2004,2005,2006,2007,2010,2011,2012,2013,2014,2015 Olly Betts
+//  Copyright (C) 2001-2004,2005,2006,2007,2010,2011,2012,2013,2014,2015,2016 Olly Betts
 //  Copyright (C) 2005 Martin Green
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -244,6 +244,10 @@ private:
 
     ZoomBox zoombox;
 
+    // Copied from parent, so we can adjust view when reloading the same
+    // file with the view restricted.
+    Vector3 offsets;
+
     // DEM:
     unsigned short * dem;
     unsigned long dem_width, dem_height;
