@@ -117,11 +117,13 @@ inline void call_lists(GLsizei n, const GLvoid * lists)
 void
 BitmapFont::init_extra_chars() const
 {
+    int fd = wxOpen(font_file,
 #ifdef __WXMSW__
-    int fd = _wopen(font_file.fn_str(), _O_RDONLY|_O_SEQUENTIAL);
+	    _O_RDONLY|_O_SEQUENTIAL|_O_BINARY,
 #else
-    int fd = open(font_file.mb_str(), O_RDONLY);
+	    O_RDONLY,
 #endif
+	    0);
 
     int data_len = 0;
     unsigned char * data = NULL;
@@ -142,7 +144,7 @@ BitmapFont::init_extra_chars() const
 	}
     }
 #else
-    data = new unsigned char [data_len];
+    data = new unsigned char[data_len];
     extra_data = data;
     if (data_len) {
 	size_t c = data_len;
@@ -166,7 +168,7 @@ BitmapFont::init_extra_chars() const
     if (fd >= 0)
 	close(fd);
 
-    extra_chars = new int [0x10000 - BITMAPFONT_MAX_CHAR];
+    extra_chars = new int[0x10000 - BITMAPFONT_MAX_CHAR];
     int data_ch = 0;
     for (int i = 0; i < 0x10000 - BITMAPFONT_MAX_CHAR; ++i) {
 	if (data_ch >= data_len) {
