@@ -4,7 +4,7 @@
 //  Handlers for events relating to the display of a survey.
 //
 //  Copyright (C) 2000-2002,2005 Mark R. Shinwell
-//  Copyright (C) 2001-2004,2006,2014 Olly Betts
+//  Copyright (C) 2001-2004,2006,2014,2015 Olly Betts
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -75,6 +75,8 @@ public:
     void OnColourByDepth();
     void OnColourByDate();
     void OnColourByError();
+    void OnColourByGradient();
+    void OnColourByLength();
     void OnShowCrosses();
     void OnShowStationNames();
     void OnShowSurveyLegs();
@@ -121,9 +123,12 @@ public:
     void OnKeyPress(wxKeyEvent &e);
 
     void OnDisplayOverlappingNamesUpdate(wxUpdateUIEvent&);
+    void OnColourByUpdate(wxUpdateUIEvent&);
     void OnColourByDepthUpdate(wxUpdateUIEvent&);
     void OnColourByDateUpdate(wxUpdateUIEvent&);
     void OnColourByErrorUpdate(wxUpdateUIEvent&);
+    void OnColourByGradientUpdate(wxUpdateUIEvent&);
+    void OnColourByLengthUpdate(wxUpdateUIEvent&);
     void OnShowCrossesUpdate(wxUpdateUIEvent&);
     void OnShowStationNamesUpdate(wxUpdateUIEvent&);
     void OnShowSurveyLegsUpdate(wxUpdateUIEvent&);
@@ -139,21 +144,9 @@ public:
     void OnToggleRotationUpdate(wxUpdateUIEvent&);
     void OnReverseControlsUpdate(wxUpdateUIEvent&);
     void OnReverseDirectionOfRotationUpdate(wxUpdateUIEvent&);
-    void OnSlowDownUpdate(wxUpdateUIEvent&);
-    void OnSpeedUpUpdate(wxUpdateUIEvent&);
-    void OnStepOnceAnticlockwiseUpdate(wxUpdateUIEvent&);
-    void OnStepOnceClockwiseUpdate(wxUpdateUIEvent&);
     void OnDefaultsUpdate(wxUpdateUIEvent&);
     void OnElevationUpdate(wxUpdateUIEvent&);
-    void OnHigherViewpointUpdate(wxUpdateUIEvent&);
-    void OnLowerViewpointUpdate(wxUpdateUIEvent&);
     void OnPlanUpdate(wxUpdateUIEvent&);
-    void OnShiftDisplayDownUpdate(wxUpdateUIEvent&);
-    void OnShiftDisplayLeftUpdate(wxUpdateUIEvent&);
-    void OnShiftDisplayRightUpdate(wxUpdateUIEvent&);
-    void OnShiftDisplayUpUpdate(wxUpdateUIEvent&);
-    void OnZoomInUpdate(wxUpdateUIEvent&);
-    void OnZoomOutUpdate(wxUpdateUIEvent&);
     void OnToggleScalebarUpdate(wxUpdateUIEvent&);
     void OnToggleColourKeyUpdate(wxUpdateUIEvent&);
     void OnViewCompassUpdate(wxUpdateUIEvent&);
@@ -198,6 +191,9 @@ public:
 
     void OnViewBoundingBoxUpdate(wxUpdateUIEvent&);
     void OnViewBoundingBox();
+
+    void OnViewTerrainUpdate(wxUpdateUIEvent&);
+    void OnViewTerrain();
 };
 
 #endif
