@@ -4,7 +4,7 @@
 //  Main frame handling for Aven.
 //
 //  Copyright (C) 2000-2002,2005,2006 Mark R. Shinwell
-//  Copyright (C) 2001-2003,2004,2005,2006,2010,2011,2012,2013,2014,2015 Olly Betts
+//  Copyright (C) 2001-2003,2004,2005,2006,2010,2011,2012,2013,2014,2015,2016 Olly Betts
 //  Copyright (C) 2005 Martin Green
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -49,6 +49,7 @@
 # include <wx/sysopt.h>
 #endif
 
+#include <cerrno>
 #include <cstdlib>
 #include <float.h>
 #include <functional>
@@ -520,6 +521,7 @@ BEGIN_EVENT_TABLE(MainFrm, wxFrame)
     EVT_MENU(menu_FILE_SCREENSHOT, MainFrm::OnScreenshot)
 //    EVT_MENU(wxID_PREFERENCES, MainFrm::OnFilePreferences)
     EVT_MENU(menu_FILE_EXPORT, MainFrm::OnExport)
+    EVT_MENU(menu_FILE_EXTEND, MainFrm::OnExtend)
     EVT_MENU(wxID_EXIT, MainFrm::OnQuit)
     EVT_MENU_RANGE(wxID_FILE1, wxID_FILE9, MainFrm::OnMRUFile)
 
@@ -608,6 +610,7 @@ BEGIN_EVENT_TABLE(MainFrm, wxFrame)
     EVT_UPDATE_UI(wxID_PRINT, MainFrm::OnPrintUpdate)
     EVT_UPDATE_UI(menu_FILE_SCREENSHOT, MainFrm::OnScreenshotUpdate)
     EVT_UPDATE_UI(menu_FILE_EXPORT, MainFrm::OnExportUpdate)
+    EVT_UPDATE_UI(menu_FILE_EXTEND, MainFrm::OnExtendUpdate)
     EVT_UPDATE_UI(menu_ROTATION_TOGGLE, MainFrm::OnToggleRotationUpdate)
     EVT_UPDATE_UI(menu_ROTATION_REVERSE, MainFrm::OnReverseDirectionOfRotationUpdate)
     EVT_UPDATE_UI(menu_ORIENT_MOVE_NORTH, MainFrm::OnMoveNorthUpdate)
@@ -782,7 +785,7 @@ void MainFrm::CreateMenuBar()
     /* TRANSLATORS: Open a "Terrain file" - i.e. a digital model of the
      * terrain. */
     filemenu->Append(menu_FILE_OPEN_TERRAIN, wmsg(/*Open &Terrain...*/453));
-    filemenu->Append(menu_FILE_LOG, wmsg(/*Show &Log*/144));
+    filemenu->AppendCheckItem(menu_FILE_LOG, wmsg(/*Show &Log*/144));
     filemenu->AppendSeparator();
     // wxID_PRINT stock label lacks the ellipses
     filemenu->Append(wxID_PRINT, wmsg(/*&Print...\tCtrl+P*/380));
@@ -791,6 +794,8 @@ void MainFrm::CreateMenuBar()
     /* TRANSLATORS: In the "File" menu */
     filemenu->Append(menu_FILE_SCREENSHOT, wmsg(/*&Screenshot...*/201));
     filemenu->Append(menu_FILE_EXPORT, wmsg(/*&Export as...*/382));
+    /* TRANSLATORS: In the "File" menu - c.f. n:191 */
+    filemenu->Append(menu_FILE_EXTEND, wmsg(/*E&xtended Elevation...*/247));
 #ifndef __WXMAC__
     // On wxMac the "Quit" menu item will be moved elsewhere, so we suppress
     // this separator.
@@ -1004,7 +1009,7 @@ void MainFrm::MakeToolBar()
     // FIXME: TRANSLATE tooltips
     toolbar->AddTool(wxID_OPEN, wxT("Open"), TOOL(open), wxT("Open a survey file for viewing"));
     toolbar->AddTool(menu_PRES_OPEN, wxT("Open presentation"), TOOL(open_pres), wxT("Open a presentation"));
-    toolbar->AddTool(menu_FILE_LOG, wxT("View log"), TOOL(log), wxT("View log from processing survey data"));
+    toolbar->AddCheckTool(menu_FILE_LOG, wxT("View log"), TOOL(log), wxNullBitmap, wxT("View log from processing survey data"));
     toolbar->AddSeparator();
     toolbar->AddCheckTool(menu_ROTATION_TOGGLE, wxT("Toggle rotation"), TOOL(rotation), wxNullBitmap, wxT("Toggle rotation"));
     toolbar->AddTool(menu_ORIENT_PLAN, wxT("Plan"), TOOL(plan), wxT("Switch to plan view"));
@@ -1513,7 +1518,13 @@ bool MainFrm::LoadData(const wxString& file, const wxString & prefix)
     m_Labels.sort(LabelCmp(separator));
 
     // Fill the tree of stations and prefixes.
-    FillTree(wxFileNameFromPath(file));
+    wxString root_name = wxFileNameFromPath(file);
+    if (!prefix.empty()) {
+	root_name += " (";
+	root_name += prefix;
+	root_name += ")";
+    }
+    FillTree(root_name);
 
     // Sort labels so that entrances are displayed in preference,
     // then fixed points, then exported points, then other points.
@@ -1528,6 +1539,8 @@ bool MainFrm::LoadData(const wxString& file, const wxString & prefix)
 	DoFind();
     }
 
+    m_FileProcessed = file;
+
     return true;
 }
 
@@ -1828,7 +1841,7 @@ void MainFrm::OpenFile(const wxString& file, const wxString& survey)
     if (!LoadData(file, survey))
 	return;
     AddToFileHistory(file);
-    InitialiseAfterLoad(file);
+    InitialiseAfterLoad(file, survey);
 
     // If aven is showing the log for a .svx file and you load a .3d file, then
     // at this point m_Log will be the log window for the .svx file, so destroy
@@ -1839,7 +1852,7 @@ void MainFrm::OpenFile(const wxString& file, const wxString& survey)
     }
 }
 
-void MainFrm::InitialiseAfterLoad(const wxString & file)
+void MainFrm::InitialiseAfterLoad(const wxString & file, const wxString & prefix)
 {
     if (m_SashPosition < 0) {
 	// Calculate sane default width for side panel.
@@ -1860,6 +1873,7 @@ void MainFrm::InitialiseAfterLoad(const wxString & file)
     bool same_file = (file == m_File);
     if (!same_file)
 	m_File = file;
+    m_Survey = prefix;
 
     wxWindow * win = NULL;
     if (m_Splitter->GetWindow2() == NULL) {
@@ -2003,7 +2017,10 @@ void MainFrm::OnOpenTerrain(wxCommandEvent&)
 
 void MainFrm::OnShowLog(wxCommandEvent&)
 {
-    if (!m_Log) return;
+    if (!m_Log) {
+	HideLog(m_Splitter->GetWindow1());
+	return;
+    }
     wxWindow * win = m_Splitter->GetWindow1();
     m_Splitter->ReplaceWindow(win, m_Log);
     win->Show(false);
@@ -2077,6 +2094,52 @@ void MainFrm::OnExport(wxCommandEvent&)
     m_Gfx->OnExport(m_File, m_Title, m_DateStamp, m_DateStamp_numeric, m_cs_proj);
 }
 
+void MainFrm::OnExtend(wxCommandEvent&)
+{
+    wxString output = m_Survey;
+    if (output.empty()) {
+	wxFileName::SplitPath(m_File, NULL, NULL, &output, NULL, wxPATH_NATIVE);
+    }
+    output += wxT("_extend.3d");
+    {
+	AvenAllowOnTop ontop(this);
+#ifdef __WXMOTIF__
+	wxString ext(wxT("*.3d"));
+#else
+	/* TRANSLATORS: Survex is the name of the software, and "3d" refers to a
+	 * file extension, so neither should be translated. */
+	wxString ext = wmsg(/*Survex 3d files*/207);
+	ext += wxT("|*.3d");
+#endif
+	wxFileDialog dlg(this, wmsg(/*Select an output filename*/319),
+			 wxString(), output, ext,
+			 wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
+	if (dlg.ShowModal() != wxID_OK) return;
+	output = dlg.GetPath();
+    }
+    wxString cmd = get_command_path(L"extend");
+    cmd = escape_for_shell(cmd, false);
+    if (!m_Survey.empty()) {
+	cmd += wxT(" --survey=");
+	cmd += escape_for_shell(m_Survey, false);
+    }
+    cmd += wxT(" --show-breaks ");
+    cmd += escape_for_shell(m_FileProcessed, true);
+    cmd += wxT(" ");
+    cmd += escape_for_shell(output, true);
+    if (wxExecute(cmd, wxEXEC_SYNC) < 0) {
+	wxString m;
+	m.Printf(wmsg(/*Couldn’t run external command: “%s”*/17), cmd.c_str());
+	m += wxT(" (");
+	m += wxString(strerror(errno), wxConvUTF8);
+	m += wxT(')');
+	wxGetApp().ReportError(m);
+	return;
+    }
+    if (LoadData(output, wxString()))
+	InitialiseAfterLoad(output, wxString());
+}
+
 void MainFrm::OnQuit(wxCommandEvent&)
 {
     if (m_PresList->Modified()) {
@@ -2509,6 +2572,7 @@ void MainFrm::OnPresStop(wxCommandEvent&)
 
 void MainFrm::OnPresExportMovie(wxCommandEvent&)
 {
+#ifdef WITH_LIBAV
     AvenAllowOnTop ontop(this);
     // FIXME : Taking the leaf of the currently loaded presentation as the
     // default might make more sense?
@@ -2522,6 +2586,9 @@ void MainFrm::OnPresExportMovie(wxCommandEvent&)
 	// Error is reported by GfxCore.
 	(void)m_Gfx->ExportMovie(dlg.GetPath());
     }
+#else
+    wxGetApp().ReportError(wxT("Movie generation support code not present"));
+#endif
 }
 
 PresentationMark MainFrm::GetPresMark(int which)
@@ -2529,6 +2596,27 @@ PresentationMark MainFrm::GetPresMark(int which)
     return m_PresList->GetPresMark(which);
 }
 
+void MainFrm::RestrictTo(const wxString & survey)
+{
+    // The station names will change, so clear the current search.
+    wxCommandEvent dummy;
+    OnHide(dummy);
+
+    wxString new_prefix;
+    if (!survey.empty()) {
+	if (!m_Survey.empty()) {
+	    new_prefix = m_Survey;
+	    new_prefix += separator;
+	}
+	new_prefix += survey;
+    }
+    // FIXME: Be more efficient to reload the processed data rather rather than
+    // potentially reprocessing.
+    if (!LoadData(m_File, new_prefix))
+	return;
+    InitialiseAfterLoad(m_File, new_prefix);
+}
+
 void MainFrm::OnOpenTerrainUpdate(wxUpdateUIEvent& event)
 {
     event.Enable(!m_File.empty());
