@@ -3,7 +3,7 @@
 //
 //  Class for writing movies from Aven.
 //
-//  Copyright (C) 2004,2011,2012,2013,2014,2015 Olly Betts
+//  Copyright (C) 2004,2011,2012,2013,2014,2015,2016 Olly Betts
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -60,6 +60,7 @@
 
 #ifdef WITH_LIBAV
 extern "C" {
+# include <libavutil/imgutils.h>
 # include <libavutil/mathematics.h>
 # include <libavformat/avformat.h>
 # include <libswscale/swscale.h>
@@ -146,10 +147,28 @@ MovieMaker::MovieMaker()
 #endif
 }
 
-bool MovieMaker::Open(const char *fnm, int width, int height)
+static int
+write_packet(void *opaque, uint8_t *buf, int buf_size) {
+    FILE * fh = (FILE*)opaque;
+    size_t res = fwrite(buf, 1, buf_size, fh);
+    return res > 0 ? res : -1;
+}
+
+#define MAX_EXTENSION_LEN 8
+
+bool MovieMaker::Open(FILE* fh, const char * ext, int width, int height)
 {
 #ifdef WITH_LIBAV
-    AVOutputFormat * fmt = av_guess_format(NULL, fnm, NULL);
+    fh_to_close = fh;
+
+    AVOutputFormat * fmt = NULL;
+    char dummy_filename[MAX_EXTENSION_LEN + 3] = "x.";
+    if (strlen(ext) <= MAX_EXTENSION_LEN) {
+	strcpy(dummy_filename + 2, ext);
+	// Pass "x." + extension to av_guess_format() to avoid having to deal
+	// with wide character filenames.
+	fmt = av_guess_format(NULL, dummy_filename, NULL);
+    }
     if (!fmt) {
 	// We couldn't deduce the output format from file extension so default
 	// to MPEG.
@@ -158,6 +177,7 @@ bool MovieMaker::Open(const char *fnm, int width, int height)
 	    averrno = MOVIE_NO_SUITABLE_FORMAT;
 	    return false;
 	}
+	strcpy(dummy_filename + 2, "mpg");
     }
     if (fmt->video_codec == AV_CODEC_ID_NONE) {
 	averrno = MOVIE_AUDIO_ONLY;
@@ -171,11 +191,7 @@ bool MovieMaker::Open(const char *fnm, int width, int height)
 	return false;
     }
     oc->oformat = fmt;
-    if (strlen(fnm) >= sizeof(oc->filename)) {
-	averrno = MOVIE_FILENAME_TOO_LONG;
-	return false;
-    }
-    strcpy(oc->filename, fnm);
+    strcpy(oc->filename, dummy_filename);
 
     /* find the video encoder */
     AVCodec *codec = avcodec_find_encoder(fmt->video_codec);
@@ -202,11 +218,19 @@ bool MovieMaker::Open(const char *fnm, int width, int height)
      * of which frame timestamps are represented. For fixed-fps content,
      * timebase should be 1/framerate and timestamp increments should be
      * identical to 1. */
+#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(55, 44, 0)
+    // Old way, which now causes deprecation warnings.
     c->time_base.den = 25; // Frames per second.
     c->time_base.num = 1;
+#else
+    video_st->time_base.den = 25; // Frames per second.
+    video_st->time_base.num = 1;
+    c->time_base = video_st->time_base;
+#endif
     c->gop_size = 12; /* emit one intra frame every twelve frames at most */
     c->pix_fmt = AV_PIX_FMT_YUV420P;
     c->rc_buffer_size = c->bit_rate * 4; // Enough for 4 seconds
+    c->rc_max_rate = c->bit_rate * 2;
     // B frames are backwards predicted - they can improve compression,
     // but may slow encoding and decoding.
     // if (c->codec_id == AV_CODEC_ID_MPEG2VIDEO) {
@@ -236,7 +260,7 @@ bool MovieMaker::Open(const char *fnm, int width, int height)
 #ifndef HAVE_AVCODEC_ENCODE_VIDEO2
     outbuf = NULL;
     if (!(oc->oformat->flags & AVFMT_RAWPICTURE)) {
-	outbuf = (unsigned char *)malloc(OUTBUF_SIZE);
+	outbuf = (unsigned char *)av_malloc(OUTBUF_SIZE);
 	if (!outbuf) {
 	    averrno = AVERROR(ENOMEM);
 	    return false;
@@ -250,7 +274,8 @@ bool MovieMaker::Open(const char *fnm, int width, int height)
 	averrno = AVERROR(ENOMEM);
 	return false;
     }
-    retval = avpicture_alloc((AVPicture *)frame, c->pix_fmt, c->width, c->height);
+    retval = av_image_alloc(frame->data, frame->linesize,
+			    c->width, c->height, c->pix_fmt, 1);
     if (retval < 0) {
 	averrno = retval;
 	return false;
@@ -262,7 +287,11 @@ bool MovieMaker::Open(const char *fnm, int width, int height)
 	abort();
     }
 
-    pixels = (unsigned char *)malloc(width * height * 6);
+    frame->format = c->pix_fmt;
+    frame->width = c->width;
+    frame->height = c->height;
+
+    pixels = (unsigned char *)av_malloc(width * height * 6);
     if (!pixels) {
 	averrno = AVERROR(ENOMEM);
 	return false;
@@ -282,9 +311,12 @@ bool MovieMaker::Open(const char *fnm, int width, int height)
     }
 
     if (!(fmt->flags & AVFMT_NOFILE)) {
-	retval = avio_open(&oc->pb, fnm, AVIO_FLAG_WRITE);
-	if (retval < 0) {
-	    averrno = retval;
+	const int buf_size = 8192;
+	void * buf = av_malloc(buf_size);
+	oc->pb = avio_alloc_context(static_cast<uint8_t*>(buf), buf_size, 1,
+				    fh, NULL, write_packet, NULL);
+	if (!oc->pb) {
+	    averrno = AVERROR(ENOMEM);
 	    return false;
 	}
     }
@@ -303,7 +335,8 @@ bool MovieMaker::Open(const char *fnm, int width, int height)
     averrno = 0;
     return true;
 #else
-    (void)fnm;
+    (void)fh;
+    (void)ext;
     (void)width;
     (void)height;
     return false;
@@ -442,6 +475,7 @@ MovieMaker::Close()
 	    int got_packet;
 	    av_init_packet(&pkt);
 	    pkt.data = NULL;
+	    pkt.size = 0;
 
 	    int ret = avcodec_encode_video2(c, &pkt, NULL, &got_packet);
 	    if (ret < 0) {
@@ -519,9 +553,9 @@ MovieMaker::release()
     if (frame) {
 	av_frame_free(&frame);
     }
-    free(pixels);
+    av_free(pixels);
     pixels = NULL;
-    free(outbuf);
+    av_free(outbuf);
     outbuf = NULL;
     av_free(sws_ctx);
     sws_ctx = NULL;
@@ -534,14 +568,18 @@ MovieMaker::release()
 	}
 
 	if (!(oc->oformat->flags & AVFMT_NOFILE)) {
-	    // Close the output file.
-	    avio_close(oc->pb);
+	    // Release the AVIOContext.
+	    av_free(oc->pb);
 	}
 
 	// Free the stream.
 	av_free(oc);
 	oc = NULL;
     }
+    if (fh_to_close) {
+	fclose(fh_to_close);
+	fh_to_close = NULL;
+    }
 }
 #endif
 
@@ -585,8 +623,6 @@ MovieMaker::get_error_string() const
 	case MOVIE_FILENAME_TOO_LONG:
 	    return "Filename too long";
     }
-    return "Unknown error";
-#else
-    return "Movie generation support code not present";
 #endif
+    return "Unknown error";
 }
