@@ -1,6 +1,6 @@
 /* printing.cc */
 /* Aven printing code */
-/* Copyright (C) 1993-2003,2004,2005,2006,2010,2011,2012,2013,2014,2015 Olly Betts
+/* Copyright (C) 1993-2003,2004,2005,2006,2010,2011,2012,2013,2014,2015,2016 Olly Betts
  * Copyright (C) 2001,2004 Philip Underwood
  *
  * This program is free software; you can redistribute it and/or modify
@@ -41,7 +41,6 @@
 #include <float.h>
 #include <limits.h>
 
-#include "debug.h" /* for SVX_ASSERT */
 #include "export.h"
 #include "filelist.h"
 #include "filename.h"
@@ -106,6 +105,7 @@ enum {
 	svx_BLANKS,
 	svx_LEGEND,
 	svx_SURFACE,
+	svx_SPLAYS,
 	svx_PLAN,
 	svx_ELEV,
 	svx_ENTS,
@@ -177,7 +177,7 @@ class svxPrintout : public wxPrintout {
     // Currently unused, but "skip blank pages" would use it.
     bool scan_for_blank_pages;
 
-    wxPen *pen_frame, *pen_cross, *pen_leg, *pen_surface_leg;
+    wxPen *pen_frame, *pen_cross, *pen_leg, *pen_surface_leg, *pen_splay;
     wxColour colour_text, colour_labels;
 
     long x_t, y_t;
@@ -227,8 +227,8 @@ BEGIN_EVENT_TABLE(svxPrintDlg, wxDialog)
     EVT_CHOICE(svx_FORMAT, svxPrintDlg::OnChange)
     EVT_TEXT(svx_SCALE, svxPrintDlg::OnChange)
     EVT_COMBOBOX(svx_SCALE, svxPrintDlg::OnChange)
-    EVT_SPINCTRL(svx_BEARING, svxPrintDlg::OnChangeSpin)
-    EVT_SPINCTRL(svx_TILT, svxPrintDlg::OnChangeSpin)
+    EVT_SPINCTRLDOUBLE(svx_BEARING, svxPrintDlg::OnChangeSpin)
+    EVT_SPINCTRLDOUBLE(svx_TILT, svxPrintDlg::OnChangeSpin)
     EVT_BUTTON(wxID_PRINT, svxPrintDlg::OnPrint)
     EVT_BUTTON(svx_EXPORT, svxPrintDlg::OnExport)
     EVT_BUTTON(wxID_CANCEL, svxPrintDlg::OnCancel)
@@ -243,6 +243,7 @@ BEGIN_EVENT_TABLE(svxPrintDlg, wxDialog)
     EVT_CHECKBOX(svx_STATIONS, svxPrintDlg::OnChange)
     EVT_CHECKBOX(svx_NAMES, svxPrintDlg::OnChange)
     EVT_CHECKBOX(svx_SURFACE, svxPrintDlg::OnChange)
+    EVT_CHECKBOX(svx_SPLAYS, svxPrintDlg::OnChange)
     EVT_CHECKBOX(svx_ENTS, svxPrintDlg::OnChange)
     EVT_CHECKBOX(svx_FIXES, svxPrintDlg::OnChange)
     EVT_CHECKBOX(svx_EXPORTS, svxPrintDlg::OnChange)
@@ -289,16 +290,16 @@ static wxString projs[] = {
 #endif
 
 static const unsigned format_info[] = {
-    LABELS|LEGS|SURF|STNS|PASG|XSECT|WALLS|MARKER_SIZE|TEXT_HEIGHT|GRID|FULL_COORDS,
-    LABELS|LEGS|SURF|STNS|PASG|XSECT|WALLS,
-    LABELS|LEGS|SURF|ENTS|FIXES|EXPORTS|PROJ|EXPORT_3D,
-    LABELS|LEGS|SURF|STNS|CENTRED,
-    LEGS|CENTRED|EXPORT_3D,
-    LABELS|LEGS|ENTS|FIXES|EXPORTS|PROJ|EXPORT_3D,
-    LABELS|LEGS|SURF,
-    LABELS|LEGS|SURF|STNS|MARKER_SIZE|GRID|SCALE,
+    LABELS|LEGS|SURF|SPLAYS|STNS|PASG|XSECT|WALLS|MARKER_SIZE|TEXT_HEIGHT|GRID|FULL_COORDS,
+    LABELS|LEGS|SURF|SPLAYS|STNS|PASG|XSECT|WALLS,
+    LABELS|LEGS|SURF|SPLAYS|ENTS|FIXES|EXPORTS|PROJ|EXPORT_3D,
+    LABELS|LEGS|SURF|SPLAYS|STNS|CENTRED,
+    LEGS|SPLAYS|CENTRED|EXPORT_3D,
+    LABELS|LEGS|SPLAYS|ENTS|FIXES|EXPORTS|PROJ|EXPORT_3D,
+    LABELS|LEGS|SURF|SPLAYS,
+    LABELS|LEGS|SURF|SPLAYS|STNS|MARKER_SIZE|GRID|SCALE,
     LABELS|ENTS|FIXES|EXPORTS|EXPORT_3D,
-    LABELS|LEGS|SURF|STNS|PASG|XSECT|WALLS|MARKER_SIZE|TEXT_HEIGHT|SCALE
+    LABELS|LEGS|SURF|SPLAYS|STNS|PASG|XSECT|WALLS|MARKER_SIZE|TEXT_HEIGHT|SCALE
 };
 
 static const char * extension[] = {
@@ -343,8 +344,8 @@ svxPrintDlg::svxPrintDlg(MainFrm* mainfrm_, const wxString & filename,
 			 const wxString & datestamp, time_t datestamp_numeric,
 			 double angle, double tilt_angle,
 			 bool labels, bool crosses, bool legs, bool surf,
-			 bool tubes, bool ents, bool fixes, bool exports,
-			 bool printing, bool close_after_)
+			 bool splays, bool tubes, bool ents, bool fixes,
+			 bool exports, bool printing, bool close_after_)
 	: wxDialog(mainfrm_, -1, wxString(printing ?
 					  /* TRANSLATORS: Title of the print
 					   * dialog */
@@ -369,6 +370,8 @@ svxPrintDlg::svxPrintDlg(MainFrm* mainfrm_, const wxString & filename,
 	show_mask |= LEGS;
     if (surf)
 	show_mask |= SURF;
+    if (splays)
+	show_mask |= SPLAYS;
     if (tubes)
 	show_mask |= XSECT|WALLS|PASG;
     if (ents)
@@ -479,7 +482,9 @@ svxPrintDlg::svxPrintDlg(MainFrm* mainfrm_, const wxString & filename,
 	wxStaticText * brg_label, * tilt_label;
 	brg_label = new wxStaticText(this, -1, wmsg(/*Bearing*/259));
 	anglebox->Add(brg_label, 0, wxALIGN_CENTER_VERTICAL|wxALIGN_LEFT|wxALL, 5);
-	m_bearing = new wxSpinCtrlDouble(this, svx_BEARING);
+	// wSP_WRAP means that you can scroll past 360 to 0, and vice versa.
+	m_bearing = new wxSpinCtrlDouble(this, svx_BEARING, wxEmptyString,
+		wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS|wxSP_WRAP);
 	m_bearing->SetRange(0.0, 360.0);
 	m_bearing->SetDigits(ANGLE_DP);
 	anglebox->Add(m_bearing, 0, wxALIGN_CENTER|wxALL, 5);
@@ -514,6 +519,10 @@ svxPrintDlg::svxPrintDlg(MainFrm* mainfrm_, const wxString & filename,
 			   wxDefaultPosition, wxDefaultSize, 0,
 			   BitValidator(&m_layout.show_mask, SURF)),
 	    0, wxALIGN_LEFT|wxALL, 2);
+    v3->Add(new wxCheckBox(this, svx_SPLAYS, wmsg(/*Spla&y Legs*/406),
+			   wxDefaultPosition, wxDefaultSize, 0,
+			   BitValidator(&m_layout.show_mask, SPLAYS)),
+	    0, wxALIGN_LEFT|wxALL, 2);
     v3->Add(new wxCheckBox(this, svx_STATIONS, wmsg(/*Crosses*/261),
 			   wxDefaultPosition, wxDefaultSize, 0,
 			   BitValidator(&m_layout.show_mask, STNS)),
@@ -708,8 +717,9 @@ svxPrintDlg::OnExport(wxCommandEvent&) {
 	try {
 	    if (!Export(dlg.GetPath(), m_layout.title,
 			m_layout.datestamp, m_layout.datestamp_numeric, mainfrm,
-			m_layout.rot, m_layout.tilt, m_layout.show_mask,
-			export_format(format_idx), input_projection.mb_str(),
+			m_layout.rot, m_layout.tilt,
+			m_layout.get_effective_show_mask(),
+			export_format(format_idx), input_projection.utf8_str(),
 			grid, text_height, marker_size, m_layout.Scale)) {
 		wxString m = wxString::Format(wmsg(/*Couldn’t write file “%s”*/402).c_str(),
 					      m_File.c_str());
@@ -784,7 +794,7 @@ svxPrintDlg::OnElevationUpdate(wxUpdateUIEvent& e) {
 }
 
 void
-svxPrintDlg::OnChangeSpin(wxSpinEvent& e) {
+svxPrintDlg::OnChangeSpin(wxSpinDoubleEvent& e) {
     SomethingChanged(e.GetId());
 }
 
@@ -810,6 +820,7 @@ svxPrintDlg::SomethingChanged(int control_id) {
 	    static const struct { int id; unsigned mask; } controls[] = {
 		{ svx_LEGS, LEGS },
 		{ svx_SURFACE, SURF },
+		{ svx_SPLAYS, SPLAYS },
 		{ svx_STATIONS, STNS },
 		{ svx_NAMES, LABELS },
 		{ svx_XSECT, XSECT },
@@ -917,10 +928,13 @@ svxPrintDlg::RecalcBounds()
     double SINT = sin(rad(m_layout.tilt));
     double COST = cos(rad(m_layout.tilt));
 
-    if (m_layout.show_mask & LEGS) {
+    int show_mask = m_layout.get_effective_show_mask();
+    if (show_mask & LEGS) {
 	list<traverse>::const_iterator trav = mainfrm->traverses_begin();
 	list<traverse>::const_iterator tend = mainfrm->traverses_end();
 	for ( ; trav != tend; ++trav) {
+	    if (trav->isSplay && !(show_mask & SPLAYS))
+		continue;
 	    vector<PointInfo>::const_iterator pos = trav->begin();
 	    vector<PointInfo>::const_iterator end = trav->end();
 	    for ( ; pos != end; ++pos) {
@@ -936,10 +950,159 @@ svxPrintDlg::RecalcBounds()
 	    }
 	}
     }
-    if (m_layout.show_mask & SURF) {
+
+    if ((show_mask & XSECT) &&
+	(m_layout.tilt == 0.0 || m_layout.tilt == 90.0 || m_layout.tilt == -90.0)) {
+	list<vector<XSect> >::const_iterator trav = mainfrm->tubes_begin();
+	list<vector<XSect> >::const_iterator tend = mainfrm->tubes_end();
+	for ( ; trav != tend; ++trav) {
+	    XSect prev_pt_v;
+	    Vector3 last_right(1.0, 0.0, 0.0);
+
+	    vector<XSect>::const_iterator i = trav->begin();
+	    vector<XSect>::size_type segment = 0;
+	    while (i != trav->end()) {
+		// get the coordinates of this vertex
+		const XSect & pt_v = *i++;
+		if (m_layout.tilt == 0.0) {
+		    Double u = pt_v.GetU();
+		    Double d = pt_v.GetD();
+
+		    if (u >= 0 || d >= 0) {
+			double x = pt_v.GetX();
+			double y = pt_v.GetY();
+			double z = pt_v.GetZ();
+			double X = x * COS - y * SIN;
+			double Y = z * COST - (x * SIN + y * COS) * SINT;
+
+			if (X > m_layout.xMax) m_layout.xMax = X;
+			if (X < m_layout.xMin) m_layout.xMin = X;
+			double U = Y + max(0.0, pt_v.GetU());
+			if (U > m_layout.yMax) m_layout.yMax = U;
+			double D = Y - max(0.0, pt_v.GetD());
+			if (D < m_layout.yMin) m_layout.yMin = D;
+		    }
+		} else {
+		    // More complex, and this duplicates the algorithm from
+		    // PlotLR() - we should try to share that, maybe via a
+		    // template.
+		    Vector3 right;
+
+		    const Vector3 up_v(0.0, 0.0, 1.0);
+
+		    if (segment == 0) {
+			assert(i != trav->end());
+			// first segment
+
+			// get the coordinates of the next vertex
+			const XSect & next_pt_v = *i;
+
+			// calculate vector from this pt to the next one
+			Vector3 leg_v = next_pt_v - pt_v;
+
+			// obtain a vector in the LRUD plane
+			right = leg_v * up_v;
+			if (right.magnitude() == 0) {
+			    right = last_right;
+			} else {
+			    last_right = right;
+			}
+		    } else if (segment + 1 == trav->size()) {
+			// last segment
+
+			// Calculate vector from the previous pt to this one.
+			Vector3 leg_v = pt_v - prev_pt_v;
+
+			// Obtain a horizontal vector in the LRUD plane.
+			right = leg_v * up_v;
+			if (right.magnitude() == 0) {
+			    right = Vector3(last_right.GetX(), last_right.GetY(), 0.0);
+			} else {
+			    last_right = right;
+			}
+		    } else {
+			assert(i != trav->end());
+			// Intermediate segment.
+
+			// Get the coordinates of the next vertex.
+			const XSect & next_pt_v = *i;
+
+			// Calculate vectors from this vertex to the
+			// next vertex, and from the previous vertex to
+			// this one.
+			Vector3 leg1_v = pt_v - prev_pt_v;
+			Vector3 leg2_v = next_pt_v - pt_v;
+
+			// Obtain horizontal vectors perpendicular to
+			// both legs, then normalise and average to get
+			// a horizontal bisector.
+			Vector3 r1 = leg1_v * up_v;
+			Vector3 r2 = leg2_v * up_v;
+			r1.normalise();
+			r2.normalise();
+			right = r1 + r2;
+			if (right.magnitude() == 0) {
+			    // This is the "mid-pitch" case...
+			    right = last_right;
+			}
+			last_right = right;
+		    }
+
+		    // Scale to unit vectors in the LRUD plane.
+		    right.normalise();
+
+		    Double l = pt_v.GetL();
+		    Double r = pt_v.GetR();
+
+		    if (l >= 0 || r >= 0) {
+			// Get the x and y coordinates of the survey station
+			double pt_X = pt_v.GetX() * COS - pt_v.GetY() * SIN;
+			double pt_Y = pt_v.GetX() * SIN + pt_v.GetY() * COS;
+
+			double X, Y;
+			if (l >= 0) {
+			    // Get the x and y coordinates of the end of the left arrow
+			    Vector3 p = pt_v - right * l;
+			    X = p.GetX() * COS - p.GetY() * SIN;
+			    Y = (p.GetX() * SIN + p.GetY() * COS);
+			} else {
+			    X = pt_X;
+			    Y = pt_Y;
+			}
+			if (X > m_layout.xMax) m_layout.xMax = X;
+			if (X < m_layout.xMin) m_layout.xMin = X;
+			if (Y > m_layout.yMax) m_layout.yMax = Y;
+			if (Y < m_layout.yMin) m_layout.yMin = Y;
+
+			if (r >= 0) {
+			    // Get the x and y coordinates of the end of the right arrow
+			    Vector3 p = pt_v + right * r;
+			    X = p.GetX() * COS - p.GetY() * SIN;
+			    Y = (p.GetX() * SIN + p.GetY() * COS);
+			} else {
+			    X = pt_X;
+			    Y = pt_Y;
+			}
+			if (X > m_layout.xMax) m_layout.xMax = X;
+			if (X < m_layout.xMin) m_layout.xMin = X;
+			if (Y > m_layout.yMax) m_layout.yMax = Y;
+			if (Y < m_layout.yMin) m_layout.yMin = Y;
+		    }
+
+		    prev_pt_v = pt_v;
+
+		    ++segment;
+		}
+	    }
+	}
+    }
+
+    if (show_mask & SURF) {
 	list<traverse>::const_iterator trav = mainfrm->surface_traverses_begin();
 	list<traverse>::const_iterator tend = mainfrm->surface_traverses_end();
 	for ( ; trav != tend; ++trav) {
+	    if (trav->isSplay && !(show_mask & SPLAYS))
+		continue;
 	    vector<PointInfo>::const_iterator pos = trav->begin();
 	    vector<PointInfo>::const_iterator end = trav->end();
 	    for ( ; pos != end; ++pos) {
@@ -955,13 +1118,13 @@ svxPrintDlg::RecalcBounds()
 	    }
 	}
     }
-    if (m_layout.show_mask & (LABELS|STNS)) {
+    if (show_mask & (LABELS|STNS)) {
 	list<LabelInfo*>::const_iterator label = mainfrm->GetLabels();
 	while (label != mainfrm->GetLabelsEnd()) {
 	    double x = (*label)->GetX();
 	    double y = (*label)->GetY();
 	    double z = (*label)->GetZ();
-	    if ((m_layout.show_mask & SURF) || (*label)->IsUnderground()) {
+	    if ((show_mask & SURF) || (*label)->IsUnderground()) {
 		double X = x * COS - y * SIN;
 		if (X > m_layout.xMax) m_layout.xMax = X;
 		if (X < m_layout.xMin) m_layout.xMin = X;
@@ -1117,7 +1280,7 @@ svxPrintout::draw_info_box()
     * mis-interpreted as us claiming copyright on the survey, so let's
     * give the website URL instead */
    MOVEMM(boxwidth + 2, 2);
-   WriteString(wxT("Survex " VERSION " - http://survex.com/"));
+   WriteString(wxT("Survex " VERSION " - https://survex.com/"));
 
    draw_scale_bar(boxwidth + 10.0, 17.0, l->PaperWidth - boxwidth - 18.0);
 }
@@ -1217,21 +1380,21 @@ make_calibration(layout *l) {
       l->xMax = l->yMax = 0.1;
       l->xMin = l->yMin = 0;
 
-      stack(l,img_MOVE, NULL, &pt);
+      stack(l, img_MOVE, NULL, &pt);
       pt.x = 0.1;
-      stack(l,img_LINE, NULL, &pt);
+      stack(l, img_LINE, NULL, &pt);
       pt.y = 0.1;
-      stack(l,img_LINE, NULL, &pt);
+      stack(l, img_LINE, NULL, &pt);
       pt.x = 0.0;
-      stack(l,img_LINE, NULL, &pt);
+      stack(l, img_LINE, NULL, &pt);
       pt.y = 0.0;
-      stack(l,img_LINE, NULL, &pt);
+      stack(l, img_LINE, NULL, &pt);
       pt.x = 0.05;
       pt.y = 0.001;
-      stack(l,img_LABEL, "10cm", &pt);
+      stack(l, img_LABEL, "10cm", &pt);
       pt.x = 0.001;
       pt.y = 0.05;
-      stack(l,img_LABEL, "10cm", &pt);
+      stack(l, img_LABEL, "10cm", &pt);
       l->Scale = 1.0;
 }
 #endif
@@ -1246,7 +1409,7 @@ svxPrintout::next_page(int *pstate, char **q, int pageLim)
    if (*pstate > 0) {
       /* doing a range */
       (*pstate)++;
-      SVX_ASSERT(*p == '-');
+      wxASSERT(*p == '-');
       p++;
       while (isspace((unsigned char)*p)) p++;
       if (sscanf(p, "%u%n", &page, &c) > 0) {
@@ -1447,15 +1610,22 @@ svxPrintout::OnPrintPage(int pageNum) {
 	draw_info_box();
     }
 
-    pdc->SetClippingRegion(x_offset, y_offset,xpPageWidth+1, ypPageDepth+1);
+    pdc->SetClippingRegion(x_offset, y_offset, xpPageWidth + 1, ypPageDepth + 1);
 
     const double Sc = 1000 / l->Scale;
 
-    if (l->show_mask & LEGS) {
-	pdc->SetPen(*pen_leg);
+    int show_mask = l->get_effective_show_mask();
+    if (show_mask & LEGS) {
 	list<traverse>::const_iterator trav = mainfrm->traverses_begin();
 	list<traverse>::const_iterator tend = mainfrm->traverses_end();
 	for ( ; trav != tend; ++trav) {
+	    if (trav->isSplay) {
+		if (!(show_mask & SPLAYS))
+		    continue;
+		pdc->SetPen(*pen_splay);
+	    } else {
+		pdc->SetPen(*pen_leg);
+	    }
 	    vector<PointInfo>::const_iterator pos = trav->begin();
 	    vector<PointInfo>::const_iterator end = trav->end();
 	    for ( ; pos != end; ++pos) {
@@ -1475,21 +1645,32 @@ svxPrintout::OnPrintPage(int pageNum) {
 	}
     }
 
-    if ((l->show_mask & XSECT) &&
+    if ((show_mask & XSECT) &&
 	(l->tilt == 0.0 || l->tilt == 90.0 || l->tilt == -90.0)) {
+	pdc->SetPen(*pen_splay);
 	list<vector<XSect> >::const_iterator trav = mainfrm->tubes_begin();
 	list<vector<XSect> >::const_iterator tend = mainfrm->tubes_end();
 	for ( ; trav != tend; ++trav) {
-	    if (l->tilt == 90.0 || l->tilt == -90.0) PlotLR(*trav);
-	    if (l->tilt == 0.0) PlotUD(*trav);
+	    if (l->tilt == 0.0) {
+		PlotUD(*trav);
+	    } else {
+		// m_layout.tilt is 90.0 or -90.0 due to check above.
+		PlotLR(*trav);
+	    }
 	}
     }
 
-    if (l->show_mask & SURF) {
-	pdc->SetPen(*pen_surface_leg);
+    if (show_mask & SURF) {
 	list<traverse>::const_iterator trav = mainfrm->surface_traverses_begin();
 	list<traverse>::const_iterator tend = mainfrm->surface_traverses_end();
 	for ( ; trav != tend; ++trav) {
+	    if (trav->isSplay) {
+		if (!(show_mask & SPLAYS))
+		    continue;
+		pdc->SetPen(*pen_splay);
+	    } else {
+		pdc->SetPen(*pen_surface_leg);
+	    }
 	    vector<PointInfo>::const_iterator pos = trav->begin();
 	    vector<PointInfo>::const_iterator end = trav->end();
 	    for ( ; pos != end; ++pos) {
@@ -1509,24 +1690,24 @@ svxPrintout::OnPrintPage(int pageNum) {
 	}
     }
 
-    if (l->show_mask & (LABELS|STNS)) {
-	if (l->show_mask & LABELS) SetFont(font_labels);
+    if (show_mask & (LABELS|STNS)) {
+	if (show_mask & LABELS) SetFont(font_labels);
 	list<LabelInfo*>::const_iterator label = mainfrm->GetLabels();
 	while (label != mainfrm->GetLabelsEnd()) {
 	    double px = (*label)->GetX();
 	    double py = (*label)->GetY();
 	    double pz = (*label)->GetZ();
-	    if ((l->show_mask & SURF) || (*label)->IsUnderground()) {
+	    if ((show_mask & SURF) || (*label)->IsUnderground()) {
 		double X = px * COS - py * SIN;
 		double Y = pz * COST - (px * SIN + py * COS) * SINT;
 		long xnew, ynew;
 		xnew = (long)((X * Sc + l->xOrg) * l->scX);
 		ynew = (long)((Y * Sc + l->yOrg) * l->scY);
-		if (l->show_mask & STNS) {
+		if (show_mask & STNS) {
 		    pdc->SetPen(*pen_cross);
 		    DrawCross(xnew, ynew);
 		}
-		if (l->show_mask & LABELS) {
+		if (show_mask & LABELS) {
 		    pdc->SetTextForeground(colour_labels);
 		    MoveTo(xnew, ynew);
 		    WriteString((*label)->GetText());
@@ -1567,6 +1748,7 @@ svxPrintout::OnBeginPrinting() {
     pen_cross = new wxPen(colour_cross);
     pen_leg = new wxPen(colour_leg);
     pen_surface_leg = new wxPen(colour_surface_leg);
+    pen_splay = new wxPen(wxColour(192, 192, 192));
 
     m_layout->scX = 1;
     m_layout->scY = 1;
@@ -1587,6 +1769,7 @@ svxPrintout::OnEndPrinting() {
     delete pen_cross;
     delete pen_leg;
     delete pen_surface_leg;
+    delete pen_splay;
 }
 
 int
@@ -1640,7 +1823,7 @@ svxPrintout::check_intersection(long x_p, long y_p)
       double v = (double)(x_p - xpPageWidth) / (x_p - x_t);
       return v >= 0 && v <= 1;
    }
-   SVX_ASSERT(mask_t & L);
+   wxASSERT(mask_t & L);
    {
       double v = (double)x_p / (x_p - x_t);
       return v >= 0 && v <= 1;
@@ -1992,25 +2175,66 @@ svxPrintout::PlotLR(const vector<XSect> & centreline)
 	Double l = pt_v.GetL();
 	Double r = pt_v.GetR();
 
-	if (l >= 0) {
-	    Vector3 p = pt_v - right * l;
-	    double X = p.GetX() * COS - p.GetY() * SIN;
-	    double Y = (p.GetX() * SIN + p.GetY() * COS);
-	    long x = (long)((X * Sc + m_layout->xOrg) * m_layout->scX);
-	    long y = (long)((Y * Sc + m_layout->yOrg) * m_layout->scY);
-	    MoveTo(x - PWX_CROSS_SIZE, y - PWX_CROSS_SIZE);
-	    DrawTo(x, y);
-	    DrawTo(x - PWX_CROSS_SIZE, y + PWX_CROSS_SIZE);
-	}
-	if (r >= 0) {
-	    Vector3 p = pt_v + right * r;
-	    double X = p.GetX() * COS - p.GetY() * SIN;
-	    double Y = (p.GetX() * SIN + p.GetY() * COS);
-	    long x = (long)((X * Sc + m_layout->xOrg) * m_layout->scX);
-	    long y = (long)((Y * Sc + m_layout->yOrg) * m_layout->scY);
-	    MoveTo(x + PWX_CROSS_SIZE, y - PWX_CROSS_SIZE);
-	    DrawTo(x, y);
-	    DrawTo(x + PWX_CROSS_SIZE, y + PWX_CROSS_SIZE);
+	if (l >= 0 || r >= 0) {
+	    // Get the x and y coordinates of the survey station
+	    double pt_X = pt_v.GetX() * COS - pt_v.GetY() * SIN;
+	    double pt_Y = pt_v.GetX() * SIN + pt_v.GetY() * COS;
+	    long pt_x = (long)((pt_X * Sc + m_layout->xOrg) * m_layout->scX);
+	    long pt_y = (long)((pt_Y * Sc + m_layout->yOrg) * m_layout->scY);
+
+	    // Calculate dimensions for the right arrow
+	    double COSR = right.GetX();
+	    double SINR = right.GetY();
+	    long CROSS_MAJOR = (COSR + SINR) * PWX_CROSS_SIZE;
+	    long CROSS_MINOR = (COSR - SINR) * PWX_CROSS_SIZE;
+
+	    if (l >= 0) {
+		// Get the x and y coordinates of the end of the left arrow
+		Vector3 p = pt_v - right * l;
+		double X = p.GetX() * COS - p.GetY() * SIN;
+		double Y = (p.GetX() * SIN + p.GetY() * COS);
+		long x = (long)((X * Sc + m_layout->xOrg) * m_layout->scX);
+		long y = (long)((Y * Sc + m_layout->yOrg) * m_layout->scY);
+
+		// Draw the arrow stem
+		MoveTo(pt_x, pt_y);
+		DrawTo(x, y);
+
+		// Rotate the arrow by the page rotation
+		long dx1 = (+CROSS_MINOR) * COS - (+CROSS_MAJOR) * SIN;
+		long dy1 = (+CROSS_MINOR) * SIN + (+CROSS_MAJOR) * COS;
+		long dx2 = (+CROSS_MAJOR) * COS - (-CROSS_MINOR) * SIN;
+		long dy2 = (+CROSS_MAJOR) * SIN + (-CROSS_MINOR) * COS;
+
+		// Draw the arrow
+		MoveTo(x + dx1, y + dy1);
+		DrawTo(x, y);
+		DrawTo(x + dx2, y + dy2);
+	    }
+
+	    if (r >= 0) {
+		// Get the x and y coordinates of the end of the right arrow
+		Vector3 p = pt_v + right * r;
+		double X = p.GetX() * COS - p.GetY() * SIN;
+		double Y = (p.GetX() * SIN + p.GetY() * COS);
+		long x = (long)((X * Sc + m_layout->xOrg) * m_layout->scX);
+		long y = (long)((Y * Sc + m_layout->yOrg) * m_layout->scY);
+
+		// Draw the arrow stem
+		MoveTo(pt_x, pt_y);
+		DrawTo(x, y);
+
+		// Rotate the arrow by the page rotation
+		long dx1 = (-CROSS_MINOR) * COS - (-CROSS_MAJOR) * SIN;
+		long dy1 = (-CROSS_MINOR) * SIN + (-CROSS_MAJOR) * COS;
+		long dx2 = (-CROSS_MAJOR) * COS - (+CROSS_MINOR) * SIN;
+		long dy2 = (-CROSS_MAJOR) * SIN + (+CROSS_MINOR) * COS;
+
+		// Draw the arrow
+		MoveTo(x + dx1, y + dy1);
+		DrawTo(x, y);
+		DrawTo(x + dx2, y + dy2);
+	    }
 	}
 
 	prev_pt_v = pt_v;
@@ -2034,24 +2258,42 @@ svxPrintout::PlotUD(const vector<XSect> & centreline)
 	Double d = pt_v.GetD();
 
 	if (u >= 0 || d >= 0) {
+	    // Get the coordinates of the survey point
 	    Vector3 p = pt_v;
 	    double SIN = sin(rad(m_layout->rot));
 	    double COS = cos(rad(m_layout->rot));
 	    double X = p.GetX() * COS - p.GetY() * SIN;
 	    double Y = p.GetZ();
 	    long x = (long)((X * Sc + m_layout->xOrg) * m_layout->scX);
+	    long pt_y = (long)((Y * Sc + m_layout->yOrg) * m_layout->scX);
+
 	    if (u >= 0) {
+		// Get the y coordinate of the up arrow
 		long y = (long)(((Y + u) * Sc + m_layout->yOrg) * m_layout->scY);
-		MoveTo(x - PWX_CROSS_SIZE, y + PWX_CROSS_SIZE);
+
+		// Draw the arrow stem
+		MoveTo(x, pt_y);
 		DrawTo(x, y);
-		DrawTo(x + PWX_CROSS_SIZE, y + PWX_CROSS_SIZE);
-	    }
-	    if (d >= 0) {
-		long y = (long)(((Y - d) * Sc + m_layout->yOrg) * m_layout->scY);
+
+		// Draw the up arrow
 		MoveTo(x - PWX_CROSS_SIZE, y - PWX_CROSS_SIZE);
 		DrawTo(x, y);
 		DrawTo(x + PWX_CROSS_SIZE, y - PWX_CROSS_SIZE);
 	    }
+
+	    if (d >= 0) {
+		// Get the y coordinate of the down arrow
+		long y = (long)(((Y - d) * Sc + m_layout->yOrg) * m_layout->scY);
+
+		// Draw the arrow stem
+		MoveTo(x, pt_y);
+		DrawTo(x, y);
+
+		// Draw the down arrow
+		MoveTo(x - PWX_CROSS_SIZE, y + PWX_CROSS_SIZE);
+		DrawTo(x, y);
+		DrawTo(x + PWX_CROSS_SIZE, y + PWX_CROSS_SIZE);
+	    }
 	}
     }
 }
